aboutsummaryrefslogtreecommitdiffstats
path: root/swarm/storage/mru/update.go
diff options
context:
space:
mode:
Diffstat (limited to 'swarm/storage/mru/update.go')
-rw-r--r--swarm/storage/mru/update.go132
1 files changed, 58 insertions, 74 deletions
diff --git a/swarm/storage/mru/update.go b/swarm/storage/mru/update.go
index d1bd37ddf..6aa57fce1 100644
--- a/swarm/storage/mru/update.go
+++ b/swarm/storage/mru/update.go
@@ -17,36 +17,35 @@
package mru
import (
- "encoding/binary"
- "errors"
+ "fmt"
+ "strconv"
"github.com/ethereum/go-ethereum/swarm/chunk"
- "github.com/ethereum/go-ethereum/swarm/log"
- "github.com/ethereum/go-ethereum/swarm/multihash"
)
-// resourceUpdate encapsulates the information sent as part of a resource update
-type resourceUpdate struct {
- updateHeader // metainformationa about this resource update
- data []byte // actual data payload
+// ProtocolVersion defines the current version of the protocol that will be included in each update message
+const ProtocolVersion uint8 = 0
+
+const headerLength = 8
+
+// Header defines a update message header including a protocol version byte
+type Header struct {
+ Version uint8 // Protocol version
+ Padding [headerLength - 1]uint8 // reserved for future use
}
-// Update chunk layout
-// Prefix:
-// 2 bytes updateHeaderLength
-// 2 bytes data length
-const chunkPrefixLength = 2 + 2
+// ResourceUpdate encapsulates the information sent as part of a resource update
+type ResourceUpdate struct {
+ Header Header //
+ ID // Resource update identifying information
+ data []byte // actual data payload
+}
-// Header: (see updateHeader)
-// Data:
-// data (datalength bytes)
-//
-// Minimum size is Header + 1 (minimum data length, enforced)
-const minimumUpdateDataLength = updateHeaderLength + 1
-const maxUpdateDataLength = chunk.DefaultSize - signatureLength - updateHeaderLength - chunkPrefixLength
+const minimumUpdateDataLength = idLength + headerLength + 1
+const maxUpdateDataLength = chunk.DefaultSize - signatureLength - idLength - headerLength
// binaryPut serializes the resource update information into the given slice
-func (r *resourceUpdate) binaryPut(serializedData []byte) error {
+func (r *ResourceUpdate) binaryPut(serializedData []byte) error {
datalength := len(r.data)
if datalength == 0 {
return NewError(ErrInvalidValue, "cannot update a resource with no data")
@@ -60,26 +59,17 @@ func (r *resourceUpdate) binaryPut(serializedData []byte) error {
return NewErrorf(ErrInvalidValue, "slice passed to putBinary must be of exact size. Expected %d bytes", r.binaryLength())
}
- if r.multihash {
- if _, _, err := multihash.GetMultihashLength(r.data); err != nil {
- return NewError(ErrInvalidValue, "Invalid multihash")
- }
- }
-
- // Add prefix: updateHeaderLength and actual data length
- cursor := 0
- binary.LittleEndian.PutUint16(serializedData[cursor:], uint16(updateHeaderLength))
- cursor += 2
+ var cursor int
+ // serialize Header
+ serializedData[cursor] = r.Header.Version
+ copy(serializedData[cursor+1:headerLength], r.Header.Padding[:headerLength-1])
+ cursor += headerLength
- // data length
- binary.LittleEndian.PutUint16(serializedData[cursor:], uint16(datalength))
- cursor += 2
-
- // serialize header (see updateHeader)
- if err := r.updateHeader.binaryPut(serializedData[cursor : cursor+updateHeaderLength]); err != nil {
+ // serialize ID
+ if err := r.ID.binaryPut(serializedData[cursor : cursor+idLength]); err != nil {
return err
}
- cursor += updateHeaderLength
+ cursor += idLength
// add the data
copy(serializedData[cursor:], r.data)
@@ -89,60 +79,54 @@ func (r *resourceUpdate) binaryPut(serializedData []byte) error {
}
// binaryLength returns the expected number of bytes this structure will take to encode
-func (r *resourceUpdate) binaryLength() int {
- return chunkPrefixLength + updateHeaderLength + len(r.data)
+func (r *ResourceUpdate) binaryLength() int {
+ return idLength + headerLength + len(r.data)
}
// binaryGet populates this instance from the information contained in the passed byte slice
-func (r *resourceUpdate) binaryGet(serializedData []byte) error {
+func (r *ResourceUpdate) binaryGet(serializedData []byte) error {
if len(serializedData) < minimumUpdateDataLength {
return NewErrorf(ErrNothingToReturn, "chunk less than %d bytes cannot be a resource update chunk", minimumUpdateDataLength)
}
- cursor := 0
- declaredHeaderlength := binary.LittleEndian.Uint16(serializedData[cursor : cursor+2])
- if declaredHeaderlength != updateHeaderLength {
- return NewErrorf(ErrCorruptData, "Invalid header length. Expected %d, got %d", updateHeaderLength, declaredHeaderlength)
- }
+ dataLength := len(serializedData) - idLength - headerLength
+ // at this point we can be satisfied that we have the correct data length to read
- cursor += 2
- datalength := int(binary.LittleEndian.Uint16(serializedData[cursor : cursor+2]))
- cursor += 2
+ var cursor int
- if chunkPrefixLength+updateHeaderLength+datalength+signatureLength != len(serializedData) {
- return NewError(ErrNothingToReturn, "length specified in header is different than actual chunk size")
- }
+ // deserialize Header
+ r.Header.Version = serializedData[cursor] // extract the protocol version
+ copy(r.Header.Padding[:headerLength-1], serializedData[cursor+1:headerLength]) // extract the padding
+ cursor += headerLength
- // at this point we can be satisfied that we have the correct data length to read
- if err := r.updateHeader.binaryGet(serializedData[cursor : cursor+updateHeaderLength]); err != nil {
+ if err := r.ID.binaryGet(serializedData[cursor : cursor+idLength]); err != nil {
return err
}
- cursor += updateHeaderLength
-
- data := serializedData[cursor : cursor+datalength]
- cursor += datalength
+ cursor += idLength
- // if multihash content is indicated we check the validity of the multihash
- if r.updateHeader.multihash {
- mhLength, mhHeaderLength, err := multihash.GetMultihashLength(data)
- if err != nil {
- log.Error("multihash parse error", "err", err)
- return err
- }
- if datalength != mhLength+mhHeaderLength {
- log.Debug("multihash error", "datalength", datalength, "mhLength", mhLength, "mhHeaderLength", mhHeaderLength)
- return errors.New("Corrupt multihash data")
- }
- }
+ data := serializedData[cursor : cursor+dataLength]
+ cursor += dataLength
// now that all checks have passed, copy data into structure
- r.data = make([]byte, datalength)
+ r.data = make([]byte, dataLength)
copy(r.data, data)
return nil
}
-// Multihash specifies whether the resource data should be interpreted as multihash
-func (r *resourceUpdate) Multihash() bool {
- return r.multihash
+// FromValues deserializes this instance from a string key-value store
+// useful to parse query strings
+func (r *ResourceUpdate) FromValues(values Values, data []byte) error {
+ r.data = data
+ version, _ := strconv.ParseUint(values.Get("protocolVersion"), 10, 32)
+ r.Header.Version = uint8(version)
+ return r.ID.FromValues(values)
+}
+
+// AppendValues serializes this structure into the provided string key-value store
+// useful to build query strings
+func (r *ResourceUpdate) AppendValues(values Values) []byte {
+ r.ID.AppendValues(values)
+ values.Set("protocolVersion", fmt.Sprintf("%d", r.Header.Version))
+ return r.data
}